@@ -25,7 +25,7 @@ from ase.calculators.calculator import FileIOCalculator, ReadError
 from ase.calculators.calculator import Parameters, all_changes
 from ase.calculators.siesta.parameters import PAOBasisBlock, Species
 from ase.calculators.siesta.parameters import format_fdf
-from collections import OrderedDict
+
 
 meV = 0.001 * eV
 
@@ -221,6 +221,15 @@ class BaseSiesta(FileIOCalculator):
         # For compatibility with old variable name:
         commandvar = os.environ.get('SIESTA_COMMAND')
         if commandvar is not None:
+            import warnings
+            warnings.warn('Please use $ASE_SIESTA_COMMAND and not '
+                          '$SIESTA_COMMAND, which will be ignored '
+                          'in the future.  The new command format will not '
+                          'work with the "<%s > %s" specification.  Use '
+                          'instead e.g. "ASE_SIESTA_COMMAND=siesta'
+                          ' < PREFIX.fdf > PREFIX.out", where PREFIX will '
+                          'automatically be replaced by calculator label',
+                          np.VisibleDeprecationWarning)
             runfile = self.prefix + '.fdf'
             outfile = self.prefix + '.out'
             try:
@@ -1071,12 +1080,16 @@ class BaseSiesta(FileIOCalculator):
 
         ksn2e = np.delete(_ee, 0, 1).reshape([nkp, nspin, n])
 
-        eig = OrderedDict()
+        eigarray = np.empty((nspin, nkp, n))
+        eigarray[:] = np.inf
+
         for k, sn2e in enumerate(ksn2e):
             for s, n2e in enumerate(sn2e):
-                eig[(k,s)] = n2e
+                eigarray[s, k, :] = n2e
 
-        self.results['eigenvalues'] = eig
+        assert np.isfinite(eigarray).all()
+
+        self.results['eigenvalues'] = eigarray
         return 0
 
     def read_kpoints(self):
@@ -1084,20 +1097,24 @@ class BaseSiesta(FileIOCalculator):
 
         fname = self.getpath(ext='KP')
         try:
-            with open(fname, "r") as f:
-                nkp = int(f.readline())
-                _ee = np.split( np.array(f.read().split()).astype(np.float), nkp)
+            with open(fname, "r") as fd:
+                nkp = int(next(fd))
+                kpoints = np.empty((nkp, 3))
+                kweights = np.empty(nkp)
+
+                for i in range(nkp):
+                    line = next(fd)
+                    tokens = line.split()
+                    numbers = np.array(tokens[1:]).astype(float)
+                    kpoints[i] = numbers[:3]
+                    kweights[i] = numbers[3]
+
         except (IOError):
             return 1
 
-        i2xyzw = np.delete(_ee, 0, 1)
-
-        kpoints, kweights = OrderedDict(), OrderedDict()
-        for i, xyzw in enumerate(i2xyzw):
-            kpoints[i], kweights[i] = xyzw[0:3], xyzw[3]
-
         self.results['kpoints'] = kpoints
         self.results['kweights'] = kweights
+
         return 0
 
     def read_dipole(self):
@@ -1601,3 +1618,12 @@ class BaseSiesta(FileIOCalculator):
             raise ValueError('units can be only au or nm**2')
 
         return data.freq, self.results['polarizability']
+
+    def get_fermi_level(self):
+        return self.results['fermi_energy']
+
+    def get_k_point_weights(self):
+        return self.results['kweights']
+
+    def get_ibz_k_points(self):
+        return self.results['kpoints']
