@@ -1,9 +1,23 @@
 import numpy as np
+from ase.utils import deprecated
 from ase.utils.arraywrapper import arraylike
 
 
 __all__ = ['Cell']
 
+# We want to deprecate the pbc keyword for Cell.
+# If it defaults to None, then the user could pass None but we wouldn't
+# know.  So we have it default to a non-None placeholder object instead:
+deprecated_placeholder = object()
+deprecation_msg = 'Cell object will no longer have pbc'
+def warn_with_pbc(array, pbc):
+    if pbc is not deprecated_placeholder:
+        import warnings
+        warnings.warn(deprecation_msg, FutureWarning)
+    if pbc is None:
+        pbc = array.any(1)
+    return pbc
+
 
 @arraylike
 class Cell:
@@ -17,29 +31,35 @@ class Cell:
 
     ase_objtype = 'cell'  # For JSON'ing
 
-    def __init__(self, array, pbc=None):
+    def __init__(self, array, pbc=deprecated_placeholder):
         """Create cell.
 
         Parameters:
 
         array: 3x3 arraylike object
-          The three cell vectors.
-        pbc: None or 3 booleans
-          For each cell vector, whether the system is periodic in the
-          direction of that cell vector.  If not given, the cell will
-          be periodic along directions with nonzero cell vectors.
+          The three cell vectors: cell[0], cell[1], and cell[2].
         """
-        if pbc is None:
-            pbc = getattr(array, 'pbc', array.any(1))
-
         array = np.asarray(array)
+        pbc = warn_with_pbc(array, pbc)
+        if pbc is deprecated_placeholder:
+            pbc = array.any(1)
 
         assert array.shape == (3, 3)
         assert array.dtype == float
         assert pbc.shape == (3,)
         assert pbc.dtype == bool
         self.array = array
-        self.pbc = pbc
+        self._pbc = pbc
+
+    @property
+    @deprecated(deprecation_msg)
+    def pbc(self):
+        return self._pbc
+
+    @pbc.setter
+    @deprecated(deprecation_msg)
+    def pbc(self, pbc):
+        self._pbc = pbc
 
     def cellpar(self, radians=False):
         """Get cell lengths and angles of this cell.
@@ -49,11 +69,7 @@ class Cell:
         return cell_to_cellpar(self.array, radians)
 
     def todict(self):
-        return dict(array=self.array, pbc=self.pbc)
-
-    @property
-    def shape(self):
-        return self.array.shape
+        return dict(array=self.array, pbc=self._pbc)
 
     @classmethod
     def ascell(cls, cell):
@@ -65,7 +81,7 @@ class Cell:
         return cls.new(cell)
 
     @classmethod
-    def new(cls, cell=None, pbc=None):
+    def new(cls, cell=None, pbc=deprecated_placeholder):
         """Create new cell from any parameters.
 
         If cell is three numbers, assume three lengths with right angles.
@@ -73,8 +89,6 @@ class Cell:
         If cell is six numbers, assume three lengths, then three angles.
 
         If cell is 3x3, assume three cell vectors."""
-        if pbc is None:
-            pbc = getattr(cell, 'pbc', None)
 
         if cell is None:
             cell = np.zeros((3, 3))
@@ -90,15 +104,12 @@ class Cell:
             raise ValueError('Cell must be length 3 sequence, length 6 '
                              'sequence or 3x3 matrix!')
 
-        cellobj = cls(cell)
-        if pbc is not None:
-            cellobj.pbc[:] = pbc
-
+        cellobj = cls(cell, pbc=pbc)
         return cellobj
 
     @classmethod
     def fromcellpar(cls, cellpar, ab_normal=(0, 0, 1), a_direction=None,
-                    pbc=None):
+                    pbc=deprecated_placeholder):
         """Return new Cell from cell lengths and angles.
 
         See also :func:`~ase.geometry.cell.cellpar_to_cell()`."""
@@ -106,7 +117,7 @@ class Cell:
         cell = cellpar_to_cell(cellpar, ab_normal, a_direction)
         return cls(cell, pbc=pbc)
 
-    def get_bravais_lattice(self, eps=2e-4):
+    def get_bravais_lattice(self, eps=2e-4, *, pbc=None):
         """Return :class:`~ase.lattice.BravaisLattice` for this cell:
 
         >>> cell = Cell.fromcellpar([4, 4, 4, 60, 60, 60])
@@ -124,11 +135,14 @@ class Cell:
            This maps the kpoints back to the original input cell.
 
         """
-        from ase.lattice import get_bravais_lattice
-        return get_bravais_lattice(self, eps=eps)
+        from ase.lattice import identify_lattice
+        if pbc is None:
+            pbc = self.any(1)
+        lat, op = identify_lattice(self, eps=eps, pbc=pbc)
+        return lat
 
     def bandpath(self, path=None, npoints=None, density=None,
-                 special_points=None, eps=2e-4):
+                 special_points=None, eps=2e-4, *, pbc=None):
         """Build a :class:`~ase.dft.kpoints.BandPath` for this cell.
 
         If special points are None, determine the Bravais lattice of
@@ -152,6 +166,10 @@ class Cell:
             For example ``{'G': [0, 0, 0], 'X': [1, 0, 0]}``.
         eps: float
             Tolerance for determining Bravais lattice.
+        pbc: three bools
+            Whether cell is periodic in each direction.  Normally not
+            necessary.  If cell has three nonzero cell vectors, use
+            e.g. pbc=[1, 1, 0] to request a 2D bandpath nevertheless.
 
         Example
         -------
@@ -161,15 +179,18 @@ class Cell:
 
         """
         # TODO: Combine with the rotation transformation from bandpath()
+
+        cell = self if pbc is None else self.uncomplete(pbc)
+
         if special_points is None:
             from ase.lattice import identify_lattice
-            lat, op = identify_lattice(self, eps=eps)
+            lat, op = identify_lattice(cell, eps=eps)
             path = lat.bandpath(path, npoints=npoints, density=density)
             return path.transform(op)
         else:
             from ase.dft.kpoints import BandPath, resolve_custom_points
             path = resolve_custom_points(path, special_points, eps=eps)
-            path = BandPath(self, path=path, special_points=special_points)
+            path = BandPath(cell, path=path, special_points=special_points)
             return path.interpolate(npoints=npoints, density=density)
 
 
@@ -181,30 +202,25 @@ class Cell:
         return bravais.bandpath(path=path, npoints=npoints, density=density,
                                 transformation=transformation)
 
+    def uncomplete(self, pbc):
+        """Return new cell, zeroing cell vectors where not periodic."""
+        pbc = np.asarray(pbc, bool)
+        cell = self.copy()
+        cell[~pbc] = 0
+        return cell
+
     def complete(self):
         """Convert missing cell vectors into orthogonal unit vectors."""
         from ase.geometry.cell import complete_cell
-        return Cell(complete_cell(self.array), self.pbc.copy())
+        cell = Cell(complete_cell(self.array))
+        cell._pbc = self._pbc.copy()
+        return cell
 
     def copy(self):
         """Return a copy of this cell."""
-        return Cell(self.array.copy(), self.pbc.copy())
-
-    @property
-    def dtype(self):
-        return self.array.dtype
-
-    @property
-    def size(self):
-        return self.array.size
-
-    @property
-    def T(self):
-        return self.array.T
-
-    @property
-    def flat(self):
-        return self.array.flat
+        cell = Cell(self.array.copy())
+        cell._pbc = self._pbc.copy()
+        return cell
 
     @property
     def rank(self):
@@ -212,26 +228,22 @@ class Cell:
 
         Equal to the number of nonzero lattice vectors."""
         # The name ndim clashes with ndarray.ndim
-        return self.array.any(1).sum()
+        return self.any(1).sum()
 
     @property
     def orthorhombic(self):
         """Return whether this cell is represented by a diagonal matrix."""
         from ase.geometry.cell import is_orthorhombic
-        return is_orthorhombic(self.array)
+        return is_orthorhombic(self)
 
     def lengths(self):
         """Return the length of each lattice vector as an array."""
-        return np.array([np.linalg.norm(v) for v in self.array])
+        return np.array([np.linalg.norm(v) for v in self])
 
     def angles(self):
         """Return an array with the three angles alpha, beta, and gamma."""
         return self.cellpar()[3:].copy()
 
-    @property
-    def ndim(self):
-        return self.array.ndim
-
     def __array__(self, dtype=float):
         if dtype != float:
             raise ValueError('Cannot convert cell to array of type {}'
@@ -239,13 +251,7 @@ class Cell:
         return self.array
 
     def __bool__(self):
-        return bool(self.array.any())
-
-    def __ne__(self, other):
-        return self.array != other
-
-    def __eq__(self, other):
-        return self.array == other
+        return bool(self.any())  # need to convert from np.bool_
 
     __nonzero__ = __bool__
 
@@ -257,10 +263,7 @@ class Cell:
         # Fail or 0 for <3D cells?
         # Definitely 0 since this is currently a property.
         # I think normally it is more convenient just to get zero
-        return np.abs(np.linalg.det(self.array))
-
-    def tolist(self):
-        return self.array.tolist()
+        return np.abs(np.linalg.det(self))
 
     def scaled_positions(self, positions):
         """Calculate scaled positions from Cartesian positions.
@@ -269,7 +272,7 @@ class Cell:
         of the cell vectors.  For the purpose of defining the basis, cell
         vectors that are zero will be replaced by unit vectors as per
         :meth:`~ase.cell.Cell.complete`."""
-        return np.linalg.solve(self.complete().array.T, positions.T).T
+        return np.linalg.solve(self.complete().T, positions.T).T
 
     def cartesian_positions(self, scaled_positions):
         """Calculate Cartesian positions from scaled positions."""
@@ -279,7 +282,7 @@ class Cell:
         """Get reciprocal lattice as a 3x3 array.
 
         Does not include factor of 2 pi."""
-        return np.linalg.pinv(self.array).transpose()
+        return np.linalg.pinv(self).transpose()
 
     def __repr__(self):
         if self.orthorhombic:
@@ -287,20 +290,15 @@ class Cell:
         else:
             numbers = self.tolist()
 
-        pbc = self.pbc
-        if all(pbc):
-            pbc = True
-        elif not any(pbc):
-            pbc = False
-        return 'Cell({}, pbc={})'.format(numbers, pbc)
+        return 'Cell({})'.format(numbers)
 
     def niggli_reduce(self, eps=1e-5):
         """Niggli reduce this cell, returning a new cell and mapping.
 
         See also :func:`ase.build.tools.niggli_reduce_cell`."""
         from ase.build.tools import niggli_reduce_cell
-        cell, op = niggli_reduce_cell(self.array, epsfactor=1e-5)
-        return Cell(cell, self.pbc), op
+        cell, op = niggli_reduce_cell(self, epsfactor=1e-5)
+        return Cell(cell), op
 
     def minkowski_reduce(self):
         """Minkowski-reduce this cell, returning new cell and mapping.
@@ -308,7 +306,7 @@ class Cell:
         See also :func:`ase.geometry.minkowski_reduction.minkowski_reduce`."""
         from ase.geometry.minkowski_reduction import minkowski_reduce
         rcell, op = minkowski_reduce(self)
-        return Cell(rcell, self.pbc), op
+        return Cell(rcell), op
 
     # XXX We want a reduction function that brings the cell into
     # standard form as defined by Setyawan and Curtarolo.
