@@ -63,11 +63,11 @@ commands = {
          '-o', '--output', '-g', '--graph', '-t', '--terminal',
          '--interpolate', '-b', '--bonds', '-s', '--scale'],
     'info':
-        ['-v', '--verbose', '--formats'],
+        ['-v', '--verbose', '--formats', '--calculators'],
     'nomad-get':
         [],
     'nomad-upload':
-        ['-t', '--token', '-n', '--do-not-save-token', '-0', '--dry-run'],
+        ['-t', '--token', '-n', '--no-save-token', '-0', '--dry-run'],
     'reciprocal':
         ['-v', '--verbose', '-p', '--path', '-d', '--dimension',
          '--no-vectors', '-k', '--k-points', '-i',
@@ -76,12 +76,10 @@ commands = {
         ['-p', '--parameters', '-t', '--tag', '--properties', '-f',
          '--maximum-force', '--constrain-tags', '-s',
          '--maximum-stress', '-E', '--equation-of-state',
-         '--eos-type', '--modify', '--after', '-i',
-         '--interactive', '-c', '--collection', '-d',
-         '--database', '-S', '--skip'],
+         '--eos-type', '-o', '--output', '--modify', '--after'],
     'test':
         ['-c', '--calculators', '--list', '--list-calculators', '-j',
-         '--jobs', '-v', '--verbose'],
+         '--jobs', '-v', '--verbose', '--strict'],
     'ulm':
         ['-n', '--index', '-d', '--delete', '-v', '--verbose']}
 # End of computer generated data
@@ -117,12 +115,25 @@ def complete(word, previous, line, point):
                      'rocksalt', 'cesiumchloride', 'fluorite', 'wurtzite']
 
     elif command == 'test':
-        if previous in ['-c', '--calculator']:
+        if previous in ['-c', '--calculators']:
             from ase.calculators.calculator import names as words
+        elif not word.startswith('-'):
+            # Suggest names of tests.  We suggest all matching tests.
+            # It might be better to autocomplete only up to directory
+            # names.
+            from ase.test.testsuite import get_tests
+            words = get_tests()
 
     return words
 
 
+if sys.version_info[0] == 2:
+    import warnings
+    warnings.warn('Command-line completion running with python2.  '
+                  'Your ASE autocompletion setup is probably outdated.  '
+                  'Please consider rerunning \'ase completion\'.')
+
+
 word, previous = sys.argv[2:]
 line = os.environ['COMP_LINE']
 point = int(os.environ['COMP_POINT'])
