@@ -1,16 +1,17 @@
+from ase.cell import Cell
 from ase.geometry.cell import (cell_to_cellpar, cellpar_to_cell,
                                crystal_structure_from_cell, complete_cell,
-                               is_orthorhombic, orthorhombic)
+                               is_orthorhombic, orthorhombic,)
 from ase.geometry.geometry import (wrap_positions,
                                    get_layers, find_mic,
                                    get_duplicate_atoms,
                                    get_angles, get_distances)
 from ase.geometry.distance import distance
+from ase.geometry.minkowski_reduction import minkowski_reduce
 
-
-__all__ = ['wrap_positions', 'complete_cell',
+__all__ = ['Cell', 'wrap_positions', 'complete_cell',
            'is_orthorhombic', 'orthorhombic',
            'get_layers', 'find_mic', 'get_duplicate_atoms',
            'cell_to_cellpar', 'cellpar_to_cell',
            'crystal_structure_from_cell', 'distance',
-           'get_angles', 'get_distances']
+           'get_angles', 'get_distances', 'minkowski_reduce']
