@@ -302,7 +302,8 @@ class GUI(View, Status):
             self.bad_plot(_('Requires 3D cell.'))
             return
 
-        kwargs = dict(cell=self.atoms.cell, vectors=True)
+        kwargs = dict(cell=self.atoms.cell.uncomplete(self.atoms.pbc),
+                      vectors=True)
         self.pipe('reciprocal', kwargs)
 
     def open(self, button=None, filename=None):
@@ -580,6 +581,7 @@ class GUI(View, Status):
 
         self.window.win.after(ms, callbackwrapper)
 
+
 def webpage():
     import webbrowser
     webbrowser.open('https://wiki.fysik.dtu.dk/ase/ase/gui/gui.html')
