@@ -4,6 +4,7 @@ from os.path import basename
 
 import numpy as np
 
+from ase.calculators.calculator import PropertyNotImplementedError
 from ase.data import atomic_numbers
 from ase.data.colors import jmol_colors
 from ase.geometry import complete_cell
@@ -18,6 +19,7 @@ GREEN = '#74DF00'
 PURPLE = '#AC58FA'
 BLACKISH = '#151515'
 
+
 def get_cell_coordinates(cell, shifted=False):
     """Get start and end points of lines segments used to draw cell."""
     nn = []
@@ -102,8 +104,8 @@ class View:
         self.velocity_vector_scale = self.config['velocity_vector_scale']
 
         # buttons
-        self.b1 = 1 # left
-        self.b3 = 3 # right
+        self.b1 = 1  # left
+        self.b3 = 3  # right
         if self.config['swap_mouse']:
             self.b1 = 3
             self.b3 = 1
@@ -127,6 +129,8 @@ class View:
 
         self.window.title = title
 
+        self.call_observers()
+
         if focus:
             self.focus()
         else:
@@ -222,7 +226,12 @@ class View:
         self.draw()
 
     def get_forces(self):
-        return self.atoms.get_forces()
+        if self.atoms.calc is not None:
+            try:
+                return self.atoms.get_forces()
+            except PropertyNotImplementedError:
+                pass
+        return np.zeros((len(self.atoms), 3))
 
     def toggle_show_forces(self, key=None):
         self.draw()
@@ -338,14 +347,15 @@ class View:
 
         colorscale, cmin, cmax = self.colormode_data
         N = len(colorscale)
+        colorswhite = colorscale + ['#ffffff']
         if cmin == cmax:
             indices = [N // 2] * len(self.atoms)
         else:
-            scalars = self.get_color_scalars()
+            scalars = np.ma.array(self.get_color_scalars())
             indices = np.clip(((scalars - cmin) / (cmax - cmin) * N +
                                0.5).astype(int),
                               0, N - 1)
-        return [colorscale[i] for i in indices]
+        return [colorswhite[i] for i in indices.filled(N)]
 
     def get_color_scalars(self, frame=None):
         if self.colormode == 'tag':
@@ -366,6 +376,10 @@ class View:
                               skin=0, self_interaction=False, bothways=True)
             nl.update(self.atoms)
             return [len(nl.get_neighbors(i)[0]) for i in range(n)]
+        else:
+            scalars = np.array(self.atoms.get_array(self.colormode),
+                               dtype=float)
+            return np.ma.array(scalars, mask=np.isnan(scalars))
 
     def get_covalent_radii(self, atoms=None):
         if atoms is None:
@@ -404,8 +418,7 @@ class View:
                 vector_arrays.append(v * 10.0 * self.velocity_vector_scale)
         if self.window['toggle-show-forces']:
             f = self.get_forces()
-            if f is not None:
-                vector_arrays.append(f * self.force_vector_scale)
+            vector_arrays.append(f * self.force_vector_scale)
 
         for array in vector_arrays:
             array[:] = np.dot(array, axes) + X[:n]
@@ -438,19 +451,25 @@ class View:
                         if (np.sum([v for v in site_occ.values()])) < 1.0:
                             fill = '#ffffff'
                             circle(fill, selected[a],
-                                    A[a, 0], A[a, 1], A[a, 0] + ra, A[a, 1] + ra)
+                                   A[a, 0], A[a, 1],
+                                   A[a, 0] + ra, A[a, 1] + ra)
                         start = 0
                         # start with the dominant species
-                        for sym, occ in sorted(site_occ.items(), key=lambda x: x[1], reverse=True):
+                        for sym, occ in sorted(site_occ.items(),
+                                               key=lambda x: x[1],
+                                               reverse=True):
                             if np.round(occ, decimals=4) == 1.0:
                                 circle(colors[a], selected[a],
-                                       A[a, 0], A[a, 1], A[a, 0] + ra, A[a, 1] + ra)
+                                       A[a, 0], A[a, 1],
+                                       A[a, 0] + ra, A[a, 1] + ra)
                             else:
                                 # jmol colors for the moment
                                 extent = 360. * occ
-                                arc(self.colors[atomic_numbers[sym]], selected[a],
+                                arc(self.colors[atomic_numbers[sym]],
+                                    selected[a],
                                     start, extent,
-                                    A[a, 0], A[a, 1], A[a, 0] + ra, A[a, 1] + ra)
+                                    A[a, 0], A[a, 1],
+                                    A[a, 0] + ra, A[a, 1] + ra)
                                 start += extent
                     except KeyError:
                         # legacy behavior
@@ -462,7 +481,7 @@ class View:
                                    A[a, 0] + ra + 4, A[a, 1] + ra + 4)
 
                         circle(colors[a], selected[a],
-                            A[a, 0], A[a, 1], A[a, 0] + ra, A[a, 1] + ra)
+                               A[a, 0], A[a, 1], A[a, 0] + ra, A[a, 1] + ra)
 
                     # Draw labels on the atoms
                     if self.labels is not None:
