@@ -44,7 +44,9 @@ _PW_TOTEN = '!    total energy'
 _PW_STRESS = 'total   stress'
 _PW_FERMI = 'the Fermi energy is'
 _PW_KPTS = 'number of k points='
-_PW_BANDS = 'End of '
+_PW_BANDS = _PW_END
+_PW_BANDSTRUCTURE = 'End of band structure calculation'
+
 
 class Namelist(OrderedDict):
     """Case insensitive dict that emulates Fortran Namelists."""
@@ -114,6 +116,7 @@ def read_espresso_out(fileobj, index=-1, results_required=True):
         _PW_FERMI: [],
         _PW_KPTS: [],
         _PW_BANDS: [],
+        _PW_BANDSTRUCTURE: [],
     }
 
     for idx, line in enumerate(pwo_lines):
@@ -137,7 +140,8 @@ def read_espresso_out(fileobj, index=-1, results_required=True):
     if results_required:
         results_indexes = sorted(indexes[_PW_TOTEN] + indexes[_PW_FORCE] +
                                  indexes[_PW_STRESS] + indexes[_PW_MAGMOM] +
-                                 indexes[_PW_BANDS])
+                                 indexes[_PW_BANDS] +
+                                 indexes[_PW_BANDSTRUCTURE])
 
         # Prune to only configurations with results data before the next
         # configuration
@@ -211,8 +215,11 @@ def read_espresso_out(fileobj, index=-1, results_required=True):
                        positions_card]
             positions = [position[1] for position in positions_card]
 
+            constraint_idx = [position[2] for position in positions_card]
+            constraint = get_constraint(constraint_idx)
+
             structure = Atoms(symbols=symbols, positions=positions, cell=cell,
-                              pbc=True)
+                              constraint=constraint, pbc=True)
 
         # Extract calculation results
         # Energy
@@ -272,36 +279,36 @@ def read_espresso_out(fileobj, index=-1, results_required=True):
                           "set verbosity='high' to print them."
 
         for kpts_index in indexes[_PW_KPTS]:
-            if image_index < kpts_index < next_index:
-                nkpts = int(pwo_lines[kpts_index].split()[4])
-                kpts_index += 2
+            nkpts = int(pwo_lines[kpts_index].split()[4])
+            kpts_index += 2
 
-                if pwo_lines[kpts_index].strip() == kpoints_warning:
-                    continue
+            if pwo_lines[kpts_index].strip() == kpoints_warning:
+                continue
 
-                # QE prints the k-points in units of 2*pi/alat
-                # with alat defined as the length of the first
-                # cell vector
-                cell = structure.get_cell()
-                alat = np.linalg.norm(cell[0])
-                ibzkpts = []
-                weights = []
-                for i in range(nkpts):
-                    l =  pwo_lines[kpts_index + i].split()
-                    weights.append(float(l[-1]))
-                    coord = map(float, [l[-6], l[-5], l[-4].strip('),')])
-                    coord = np.array(coord) * 2 * np.pi / alat
-                    coord = kpoint_convert(cell, ckpts_kv=coord)
-                    ibzkpts.append(coord)
-                ibzkpts = np.array(ibzkpts)
-                weights = np.array(weights)
+            # QE prints the k-points in units of 2*pi/alat
+            # with alat defined as the length of the first
+            # cell vector
+            cell = structure.get_cell()
+            alat = np.linalg.norm(cell[0])
+            ibzkpts = []
+            weights = []
+            for i in range(nkpts):
+                l = pwo_lines[kpts_index + i].split()
+                weights.append(float(l[-1]))
+                coord = np.array([l[-6], l[-5], l[-4].strip('),')],
+                                 dtype=float)
+                coord *= 2 * np.pi / alat
+                coord = kpoint_convert(cell, ckpts_kv=coord)
+                ibzkpts.append(coord)
+            ibzkpts = np.array(ibzkpts)
+            weights = np.array(weights)
 
         # Bands
         kpts = None
         kpoints_warning = "Number of k-points >= 100: " + \
                           "set verbosity='high' to print the bands."
 
-        for bands_index in indexes[_PW_BANDS]:
+        for bands_index in indexes[_PW_BANDS] + indexes[_PW_BANDSTRUCTURE]:
             if image_index < bands_index < next_index:
                 bands_index += 2
 
@@ -312,7 +319,7 @@ def read_espresso_out(fileobj, index=-1, results_required=True):
                 spin, bands, eigenvalues = 0, [], [[], []]
 
                 while True:
-                    l = pwo_lines[bands_index].split()
+                    l = pwo_lines[bands_index].replace('-', ' -').split()
                     if len(l) == 0:
                         if len(bands) > 0:
                             eigenvalues[spin].append(bands)
@@ -321,8 +328,8 @@ def read_espresso_out(fileobj, index=-1, results_required=True):
                         bands_index += 3
                     elif l[0] == 'k' and l[1].startswith('='):
                         pass
-                    elif len(l) > 2 and l[1] == 'SPIN':
-                        if l[2] == 'DOWN':
+                    elif 'SPIN' in l:
+                        if 'DOWN' in l:
                             spin += 1
                     else:
                         try:
@@ -333,7 +340,7 @@ def read_espresso_out(fileobj, index=-1, results_required=True):
 
                 if spin == 1:
                     assert len(eigenvalues[0]) == len(eigenvalues[1])
-                assert len(eigenvalues[0]) == len(ibzkpts)
+                assert len(eigenvalues[0] + eigenvalues[1]) == len(ibzkpts)
 
                 kpts = []
                 for s in range(spin + 1):
@@ -342,8 +349,8 @@ def read_espresso_out(fileobj, index=-1, results_required=True):
                         kpts.append(kpt)
 
         # Put everything together
-        calc = SinglePointDFTCalculator(structure, energy=energy, 
-                                        forces=forces, stress=stress, 
+        calc = SinglePointDFTCalculator(structure, energy=energy,
+                                        forces=forces, stress=stress,
                                         magmoms=magmoms, efermi=efermi,
                                         ibzkpts=ibzkpts)
         calc.kpts = kpts
@@ -475,10 +482,13 @@ def read_espresso_in(fileobj):
 
     symbols = [label_to_symbol(position[0]) for position in positions_card]
     positions = [position[1] for position in positions_card]
+    constraint_idx = [position[2] for position in positions_card]
+    constraint = get_constraint(constraint_idx)
 
     # TODO: put more info into the atoms object
-    # e.g magmom, force constraints
-    atoms = Atoms(symbols=symbols, positions=positions, cell=cell, pbc=True)
+    # e.g magmom, forces.
+    atoms = Atoms(symbols=symbols, positions=positions, cell=cell,
+                  constraint=constraint, pbc=True)
 
     return atoms
 
@@ -1624,9 +1634,10 @@ def write_espresso_in(fd, atoms, input_data=None, pseudopotentials=None,
     if isinstance(koffset, int):
         koffset = (koffset, ) * 3
 
-    if isinstance(kgrid, dict):
+    # BandPath object or bandpath-as-dictionary:
+    if isinstance(kgrid, dict) or hasattr(kgrid, 'kpts'):
         pwi.append('K_POINTS crystal_b\n')
-        assert 'path' in kgrid
+        assert hasattr(kgrid, 'path') or 'path' in kgrid
         kgrid = kpts2ndarray(kgrid, atoms=atoms)
         pwi.append('%s\n' % len(kgrid))
         for k in kgrid:
@@ -1658,3 +1669,21 @@ def write_espresso_in(fd, atoms, input_data=None, pseudopotentials=None,
 
     # DONE!
     fd.write(''.join(pwi))
+
+
+def get_constraint(constraint_idx):
+    """
+    Map constraints from QE input/output to FixAtoms or FixCartesian constraint
+    """
+    if not np.any(constraint_idx):
+        return None
+
+    a = [a for a, c in enumerate(constraint_idx) if np.all(c is not None)]
+    mask = [[(ic + 1) % 2 for ic in c] for c in constraint_idx
+            if np.all(c is not None)]
+
+    if np.all(np.array(mask)) == 1:
+        constraint = FixAtoms(a)
+    else:
+        constraint = FixCartesian(a, mask)
+    return constraint
