@@ -56,8 +56,9 @@ all_formats = {
     'castep-md': ('CASTEP molecular dynamics file', '+F'),
     'castep-phonon': ('CASTEP phonon file', '1F'),
     'cfg': ('AtomEye configuration', '1F'),
-    'cif': ('CIF-file', '+F'),
+    'cif': ('CIF-file', '+B'),
     'cmdft': ('CMDFT-file', '1F'),
+    'cp2k-dcd': ('CP2K DCD file', '+B'),
     'crystal': ('Crystal fort.34 format', '1S'),
     'cube': ('CUBE file', '1F'),
     'dacapo': ('Dacapo netCDF output file', '1F'),
@@ -65,11 +66,12 @@ all_formats = {
     'db': ('ASE SQLite database file', '+S'),
     'dftb': ('DftbPlus input file', '1S'),
     'dlp4': ('DL_POLY_4 CONFIG file', '1F'),
+    'dlp-history': ('DL_POLY HISTORY file', '+F'),
     'dmol-arc': ('DMol3 arc file', '+S'),
     'dmol-car': ('DMol3 structure file', '1S'),
     'dmol-incoor': ('DMol3 structure file', '1S'),
     'elk': ('ELK atoms definition', '1S'),
-    'eon': ('EON reactant.con file', '1F'),
+    'eon': ('EON CON file', '+F'),
     'eps': ('Encapsulated Postscript', '1S'),
     'espresso-in': ('Quantum espresso in file', '1F'),
     'espresso-out': ('Quantum espresso out file', '+F'),
@@ -79,6 +81,8 @@ all_formats = {
     'findsym': ('FINDSYM-format', '+F'),
     'gaussian': ('Gaussian com (input) file', '1S'),
     'gaussian-out': ('Gaussian output file', '1F'),
+    'acemolecule-out': ('ACE output file', '1S'),
+    'acemolecule-input':('ACE input file', '1S'),
     'gen': ('DFTBPlus GEN format', '1F'),
     'gif': ('Graphics interchange format', '+S'),
     'gpaw-out': ('GPAW text output', '+F'),
@@ -95,6 +99,7 @@ all_formats = {
     'mol': ('MDL Molfile', '1F'),
     'mp4': ('MP4 animation', '+S'),
     'mustem': ('muSTEM xtl file', '1F'),
+    'mysql': ('ASE MySQL database file', '+S'),
     'netcdftrajectory': ('AMBER NetCDF trajectory file', '+S'),
     'nomad-json': ('JSON from Nomad archive', '+F'),
     'nwchem': ('NWChem input file', '1F'),
@@ -116,13 +121,14 @@ all_formats = {
     'v-sim': ('V_Sim ascii file', '1F'),
     'vasp': ('VASP POSCAR/CONTCAR file', '1F'),
     'vasp-out': ('VASP OUTCAR file', '+F'),
-    'vasp-xdatcar': ('VASP XDATCAR file', '+S'),
+    'vasp-xdatcar': ('VASP XDATCAR file', '+F'),
     'vasp-xml': ('VASP vasprun.xml file', '+F'),
     'vti': ('VTK XML Image Data', '1F'),
     'vtu': ('VTK XML Unstructured Grid', '1F'),
     'x3d': ('X3D', '1S'),
     'xsd': ('Materials Studio file', '1F'),
     'xsf': ('XCrySDen Structure File', '+F'),
+    'xtd': ('Materials Studio file', '+F'),
     'xyz': ('XYZ-file', '+F')}
 
 # Special cases:
@@ -136,17 +142,22 @@ format2modulename = {
     'castep-geom': 'castep',
     'castep-md': 'castep',
     'castep-phonon': 'castep',
+    'cp2k-dcd': 'cp2k',
     'dacapo-text': 'dacapo',
     'dlp4': 'dlp4',
+    'dlp-history': 'dlp4',
     'espresso-in': 'espresso',
     'espresso-out': 'espresso',
     'gaussian-out': 'gaussian',
+    'acemolecule-out': 'acemolecule',
+    'acemolecule-input': 'acemolecule',
     'gif': 'animation',
     'html': 'x3d',
     'json': 'db',
     'lammps-dump': 'lammpsrun',
     'lammps-data': 'lammpsdata',
     'mp4': 'animation',
+    'mysql': 'db',
     'postgresql': 'db',
     'struct': 'wien2k',
     'struct_out': 'siesta',
@@ -167,6 +178,7 @@ extension2format = {
     'com': 'gaussian',
     'con': 'eon',
     'config': 'dlp4',
+    'dcd': 'cp2k-dcd',
     'exi': 'exciting',
     'f34': 'crystal',
     '34': 'crystal',
@@ -203,8 +215,8 @@ def initialize(format):
     try:
         module = import_module('ase.io.' + module_name)
     except ImportError as err:
-        raise ValueError('File format not recognized: %s.  Error: %s'
-                         % (format, err))
+        raise UnknownFileTypeError('File format not recognized: %s.  Error: %s'
+                                   % (format, err))
 
     read = getattr(module, 'read_' + _format, None)
     write = getattr(module, 'write_' + _format, None)
@@ -212,7 +224,7 @@ def initialize(format):
     if read and not inspect.isgeneratorfunction(read):
         read = functools.partial(wrap_read_function, read)
     if not read and not write:
-        raise ValueError('File format not recognized: ' + format)
+        raise UnknownFileTypeError('File format not recognized: ' + format)
     code = all_formats[format][1]
     single = code[0] == '1'
     assert code[1] in 'BFS'
@@ -359,8 +371,9 @@ def write(filename, images, format=None, parallel=True, append=False,
         Default is to write on master only.  Use parallel=False to write
         from all slaves.
     append: bool
-        Default is to open files in 'w' or 'wb' mode, overwriting existing files.
-        In some cases opening the file in 'a' or 'ab' mode (appending) is usefull,
+        Default is to open files in 'w' or 'wb' mode, overwriting
+        existing files.  In some cases opening the file in 'a' or 'ab'
+        mode (appending) is usefull,
         e.g. writing trajectories or saving multiple Atoms objects in one file.
         WARNING: If the file format does not support multiple entries without
         additional keywords/headers, files created using 'append=True'
@@ -369,6 +382,9 @@ def write(filename, images, format=None, parallel=True, append=False,
 
     The use of additional keywords is format specific."""
 
+    if isinstance(filename, PurePath):
+        filename = str(filename)
+
     if isinstance(filename, basestring):
         filename = os.path.expanduser(filename)
         fd = None
@@ -385,11 +401,13 @@ def write(filename, images, format=None, parallel=True, append=False,
 
     io = get_ioformat(format)
 
-    _write(filename, fd, format, io, images, parallel=parallel, append=append, **kwargs)
+    _write(filename, fd, format, io, images, parallel=parallel, append=append,
+           **kwargs)
 
 
 @parallel_function
-def _write(filename, fd, format, io, images, parallel=None, append=False, **kwargs):
+def _write(filename, fd, format, io, images, parallel=None, append=False,
+           **kwargs):
     if isinstance(images, Atoms):
         images = [images]
 
@@ -403,11 +421,11 @@ def _write(filename, fd, format, io, images, parallel=None, append=False, **kwar
         raise ValueError("Can't write to {}-format".format(format))
 
     # Special case for json-format:
-    if format == 'json' and len(images) > 1:
+    if format == 'json' and (len(images) > 1 or append):
         if filename is not None:
-            io.write(filename, images, **kwargs)
+            io.write(filename, images, append=append, **kwargs)
             return
-        raise ValueError("Can't write more than one image to file-descriptor"
+        raise ValueError("Can't write more than one image to file-descriptor "
                          'using json-format.')
 
     if io.acceptsfd:
@@ -428,7 +446,8 @@ def _write(filename, fd, format, io, images, parallel=None, append=False, **kwar
             io.write(filename, images, append=append, **kwargs)
         elif append:
             raise ValueError("Cannot append to {}-format, write-function "
-                             "does not support the append keyword.".format(format))
+                             "does not support the append keyword."
+                             .format(format))
         else:
             io.write(filename, images, **kwargs)
 
@@ -603,6 +622,9 @@ def filetype(filename, read=True, guess=True):
         if filename.startswith('postgres'):
             return 'postgresql'
 
+        if filename.startswith('mysql') or filename.startswith('mariadb'):
+            return 'mysql'
+
         # strip any compression extensions that can be read
         root, compression = get_compression(filename)
         basename = os.path.basename(root)
@@ -640,6 +662,8 @@ def filetype(filename, read=True, guess=True):
             return 'iwm'
         if 'CONFIG' in basename:
             return 'dlp4'
+        if basename == 'HISTORY':
+            return 'dlp-history'
 
         if not read:
             if ext is None:
@@ -721,3 +745,38 @@ def filetype(filename, read=True, guess=True):
         raise UnknownFileTypeError('Could not guess file type')
 
     return format
+
+
+def index2range(index, nsteps):
+    """Method to convert a user given *index* option to a list of indices.
+
+    Returns a range.
+    """
+    if isinstance(index, int):
+        if index < 0:
+            tmpsnp = nsteps + index
+            trbl = range(tmpsnp, tmpsnp + 1, 1)
+        else:
+            trbl = range(index, index + 1, 1)
+    elif isinstance(index, slice):
+        start = index.start
+        stop = index.stop
+        step = index.step
+
+        if start is None:
+            start = 0
+        elif start < 0:
+            start = nsteps + start
+
+        if step is None:
+            step = 1
+
+        if stop is None:
+            stop = nsteps
+        elif stop < 0:
+            stop = nsteps + stop
+
+        trbl = range(start, stop, step)
+    else:
+        raise RuntimeError("index2range handles integers and slices only.")
+    return trbl
