@@ -89,8 +89,7 @@ class BravaisLattice(ABC):
     def tocell(self):
         """Return this lattice as a :class:`~ase.cell.Cell` object."""
         cell = self._cell(**self._parameters)
-        pbc = np.arange(3) < self.ndim
-        return Cell(cell, pbc=pbc)
+        return Cell(cell)
 
     def get_transformation(self, cell):
         # Get transformation matrix relating input cell to canonical cell
@@ -951,8 +950,14 @@ class OBL(BravaisLattice):
 
     def _special_points(self, a, b, alpha, variant):
         # XXX Check me
-        cosa = np.cos(alpha * _degrees)
-        eta = (1 - a * cosa / b) / (2 * np.sin(alpha * _degrees)**2)
+        if alpha > 90:
+            _alpha = 180 - alpha
+            a, b = b, a
+        else:
+            _alpha = alpha
+
+        cosa = np.cos(_alpha * _degrees)
+        eta = (1 - a * cosa / b) / (2 * np.sin(_alpha * _degrees)**2)
         nu = .5 - eta * b * cosa / a
 
         points = [[0, 0, 0],
@@ -961,6 +966,14 @@ class OBL(BravaisLattice):
                   [.5, .5, 0],
                   [1 - eta, nu, 0],
                   [.5, 0, 0]]
+
+        if alpha > 90:
+            # Then we map the special points back
+            op = np.array([[0, 1, 0],
+                           [-1, 0, 0],
+                           [0, 0, 1]])
+            points = np.dot(points, op.T)
+
         return points
 
 
@@ -1009,15 +1022,27 @@ class CRECT(BravaisLattice):
                          [0, 0, 0.]])
 
     def _special_points(self, a, alpha, variant):
-        sina2 = np.sin(alpha / 2 * _degrees)**2
-        sina = np.sin(alpha * _degrees)**2
+        if alpha > 90:
+            _alpha = 180 - alpha
+        else:
+            _alpha = alpha
+        sina2 = np.sin(_alpha / 2 * _degrees)**2
+        sina = np.sin(_alpha * _degrees)**2
         eta = sina2 / sina
-        cosa = np.cos(alpha * _degrees)
+        cosa = np.cos(_alpha * _degrees)
         xi = eta * cosa
+
         points = [[0, 0, 0],
                   [eta, - eta, 0],
                   [0.5 + xi, 0.5 - xi, 0],
                   [0.5, 0.5, 0]]
+
+        if alpha > 90:
+            # Then we map the special points back
+            op = np.array([[0, 1, 0],
+                           [-1, 0, 0],
+                           [0, 0, 1]])
+            points = np.dot(points, op.T)
         return points
 
 
@@ -1035,20 +1060,6 @@ class SQR(BravaisLattice):
                          [0, 0, 0.]])
 
 
-def get_bravais_lattice(cell, eps=2e-4):
-    cell = Cell.ascell(cell)
-
-    if cell.pbc.all():
-        lat, op = identify_lattice(cell, eps=eps)
-        return lat
-    elif cell.pbc[:2].all():
-        return get_2d_bravais_lattice(cell, eps)
-    else:
-        raise ValueError('Cell must be periodic either along two first '
-                         'axes or along all three.  Got pbc={}'
-                         .format(cell.pbc))
-
-
 def celldiff(cell1, cell2):
     """Return a unitless measure of the difference between two cells."""
     cell1 = Cell.ascell(cell1).complete()
@@ -1074,14 +1085,30 @@ def get_lattice_from_canonical_cell(cell, eps=2e-4):
     return LatticeChecker(cell, eps).match()
 
 
-def identify_lattice(cell, eps=2e-4):
+def identify_lattice(cell, eps=2e-4, *, pbc=None):
     """Find Bravais lattice representing this cell.
 
     Returns Bravais lattice object representing the cell along with
     and operation that, applied to the cell, yields the same lengths
     and angles as the Bravais lattice object."""
+
+    if pbc is None:
+        pbc = cell.any(1)
+
+    npbc = sum(pbc)
+    if npbc == 2:
+        lat, op = get_2d_bravais_lattice(cell, eps, pbc=pbc)
+        return lat, op
+
+    if npbc != 3:
+        raise ValueError('System must be periodic either along two first '
+                         'axes or along all three.  Got pbc={}'
+                         .format(pbc))
+
+
     from ase.geometry.bravais_type_engine import niggli_op_table
-    if not cell.volume:
+
+    if cell.rank < 3:
         raise ValueError('Expected 3 linearly independent cell vectors')
     rcell, reduction_op = cell.niggli_reduce()
 
@@ -1292,9 +1319,19 @@ class LatticeChecker:
         return self._check(TRI, *self.cellpar)
 
 
-def get_2d_bravais_lattice(origcell, eps=2e-4):
-    pbc = origcell.pbc
-    assert pbc.sum() == 2
+class UnsupportedLattice(ValueError):
+    pass
+
+
+def get_2d_bravais_lattice(origcell, eps=2e-4, *, pbc=None):
+    if pbc is None:
+        pbc = origcell.any(1)
+    pbc = np.asarray(pbc, bool)
+
+    if list(pbc) != [1, 1, 0]:
+        raise UnsupportedLattice('Can only get 2D Bravais lattice of cell with '
+                                 'pbc==[1, 1, 0]; but we have {}'.format(pbc))
+
     nonperiodic = pbc.argmin()
     # Start with op = I
     ops = [np.eye(3)]
@@ -1313,12 +1350,12 @@ def get_2d_bravais_lattice(origcell, eps=2e-4):
 
     symrank = 0
     for op in ops:
-        cell = Cell(op.dot(origcell), pbc=pbc)
+        cell = Cell(op.dot(origcell))
         cellpar = cell.cellpar()
         angles = cellpar[3:]
         # Find a, b and gamma
         gamma = angles[~pbc][0]
-        a, b = cellpar[:3][cell.pbc]
+        a, b = cellpar[:3][pbc]
 
         anglesm90 = np.abs(angles - 90)
         # Maximum one angle different from 90 deg in 2d please
@@ -1352,10 +1389,11 @@ def get_2d_bravais_lattice(origcell, eps=2e-4):
                    format(a, b, gamma))
             raise RuntimeError(msg)
         if rank > symrank:
+            finalop = op
             symrank = rank
             finallat = lat
 
-    return finallat
+    return finallat, finalop.T
 
 
 def all_variants():
