@@ -7,18 +7,11 @@ Author: Ole Schuett <ole.schuett@mat.ethz.ch>
 """
 
 from __future__ import division, print_function
-
-import os
-
-from ase.test import NotAvailable
 from ase.build import molecule
 from ase.calculators.cp2k import CP2K
 
 
 def main():
-    if "ASE_CP2K_COMMAND" not in os.environ:
-        raise NotAvailable('$ASE_CP2K_COMMAND not defined')
-
     calc = CP2K(xc='PBE', label='test_H2_PBE')
     h2 = molecule('H2', calculator=calc)
     h2.center(vacuum=2.0)
