@@ -8,19 +8,19 @@ from ase import Atoms
 from ase.calculators.singlepoint import SinglePointCalculator
 from ase.build import molecule
 from ase.gui.i18n import _
-from ase.test import NotAvailable
+import unittest
 
 try:
     import ase.gui.ui as ui
 except ImportError:
-    raise NotAvailable
+    raise unittest.SkipTest
 
 from ase.gui.gui import GUI
 from ase.gui.save import save_dialog
 
 
 if not os.environ.get('DISPLAY'):
-    raise NotAvailable
+    raise unittest.SkipTest
 
 
 class Error:
@@ -83,13 +83,18 @@ def color(gui):
     a = Atoms('C10', magmoms=np.linspace(1, -1, 10))
     a.positions[:] = np.linspace(0, 9, 10)[:, None]
     a.calc = SinglePointCalculator(a, forces=a.positions)
+    che = np.linspace(100, 110, 10)
+    mask = [0] * 10
+    mask[5] = 1
+    a.set_array('corehole_energies', np.ma.array(che, mask=mask))
     gui.new_atoms(a)
     c = gui.colors_window()
     c.toggle('force')
-    text = c.toggle('magmom')
+    c.toggle('magmom')
     activebuttons = [button.active for button in c.radio.buttons]
-    assert activebuttons == [1, 0, 1, 0, 0, 1, 1], activebuttons
-    assert text.rsplit('[', 1)[1].startswith('-1.000000,1.000000]')
+    assert activebuttons == [1, 0, 1, 0, 0, 1, 1, 1], activebuttons
+    c.toggle('corehole_energies')
+    c.change_mnmx(101, 120)
 
 
 @test
@@ -115,6 +120,7 @@ def open_and_save(gui):
     gui.open(filename='h2o.json')
     save_dialog(gui, 'h2o.cif@-1')
 
+
 @test
 def test_fracocc(gui):
     from ase.test.fio.cif import content
@@ -123,7 +129,6 @@ def test_fracocc(gui):
     gui.open(filename='fracocc.cif')
 
 
-
 p = argparse.ArgumentParser()
 p.add_argument('tests', nargs='*')
 p.add_argument('-p', '--pause', action='store_true')
@@ -152,21 +157,19 @@ for name in args.tests or alltests:
     gui.run(test=f)
 
 
-
-
 import os
 from functools import partial
 
-from ase.test import NotAvailable
+import unittest
 
 try:
     import ase.gui.ui as ui
 except ImportError:
-    raise NotAvailable
+    raise unittest.SkipTest
 
 
 if not os.environ.get('DISPLAY'):
-    raise NotAvailable
+    raise unittest.SkipTest
 
 
 def window():
