@@ -87,9 +87,6 @@ def main():
     txt = git('status')
     branch = re.match('On branch (\S+)', txt).group(1)
     print('Currently on branch {}'.format(repr(branch)))
-    if branch != 'master':
-        git('checkout master')
-
 
     git('checkout -b {}'.format(branchname))
 
@@ -181,10 +178,10 @@ News
     with open(installdoc) as fd:
         txt = fd.read()
 
-    txt, nsub = re.subn(r'ase-\d+\.\d+.\d+',
+    txt, nsub = re.subn(r'ase-\d+\.\d+\.\d+',
                         'ase-{}'.format(version), txt)
     assert nsub > 0
-    txt, nsub = re.subn(r'git clone -b \d+\.\d+.\d+',
+    txt, nsub = re.subn(r'git clone -b \d+\.\d+\.\d+',
                         'git clone -b {}'.format(version), txt)
     assert nsub == 1
 
