@@ -1,7 +1,6 @@
 # creates: Al_phonon.png, Al_mode.gif
 from ase.build import bulk
 from ase.calculators.emt import EMT
-from ase.dft.kpoints import bandpath
 from ase.phonons import Phonons
 
 # Setup crystal and EMT calculator
@@ -16,7 +15,7 @@ ph.run()
 ph.read(acoustic=True)
 ph.clean()
 
-path = bandpath('GXULGK', atoms.cell, 100)[0]
+path = atoms.cell.bandpath('GXULGK', npoints=100)
 bs = ph.get_band_structure(path)
 
 dos = ph.get_dos(kpts=(20, 20, 20)).sample_grid(npts=100, width=1e-3)
@@ -40,29 +39,16 @@ dosax.set_xlabel("DOS", fontsize=18)
 
 fig.savefig('Al_phonon.png')
 
-# --- End of literalinclude ---
-
-# Literalinclude start modes
-# Write modes for specific q-vector to trajectory files
-from ase.dft.kpoints import ibz_points
-L = ibz_points['fcc']['L']
-ph.write_modes([l / 2 for l in L], branches=[2], repeat=(8, 8, 8), kT=3e-4,
-               center=True)
-# Literalinclude end modes
-
-
-# Generate png animation (requires the convert command from ImageMagick):
-from subprocess import call
+# --- literalinclude division line ---
 from ase.io.trajectory import Trajectory
 from ase.io import write
-trajectory = Trajectory('phonon.mode.2.traj', 'r')
 
-for i, atoms in enumerate(trajectory):
-    write('picture%02i.png' % i, atoms, show_unit_cell=2,
+# Write modes for specific q-vector to trajectory files:
+L = path.special_points['L']
+ph.write_modes([l / 2 for l in L], branches=[2], repeat=(8, 8, 8), kT=3e-4,
+               center=True)
+
+# Generate gif animation:
+with Trajectory('phonon.mode.2.traj', 'r') as traj:
+    write('Al_mode.gif', traj, interval=50,
           rotation='-36x,26.5y,-25z')
-    # Flatten images for better quality
-    call(['convert', '-flatten', 'picture%02i.png' % i, 'picture%02i.png' % i])
-
-# Concatenate to gif animation
-call(['convert', '-delay', '5', '-loop', '0', '-dispose', 'Previous',
-      'picture*.png', 'Al_mode.gif'])
